@@ -32,7 +32,7 @@
 #endif
 
 #define R__HAS_SETRESUID   /**/
-#undef R__HAS_MATHMORE   /**/
+#define R__HAS_MATHMORE   /**/
 #define R__HAS_PTHREAD    /**/
 #define R__HAS_XFT    /**/
 #undef R__HAS_COCOA    /**/
@@ -41,14 +41,6 @@
 #undef R__HAS_VECCORE    /**/
 #define R__USE_CXXMODULES   /**/
 #undef R__USE_LIBCXX    /**/
-#define R__HAS_STD_STRING_VIEW   /**/
-#undef R__CUDA_HAS_STD_STRING_VIEW   /**/
-#undef R__HAS_STD_EXPERIMENTAL_STRING_VIEW   /**/
-#undef R__HAS_STOD_STRING_VIEW /**/
-#define R__HAS_OP_EQUAL_PLUS_STRING_VIEW /**/
-#define R__HAS_STD_APPLY /**/
-#define R__HAS_STD_INVOKE /**/
-#define R__HAS_STD_INDEX_SEQUENCE /**/
 #define R__HAS_ATTRIBUTE_ALWAYS_INLINE /**/
 #define R__HAS_ATTRIBUTE_NOINLINE /**/
 #undef R__HAS_HARDWARE_INTERFERENCE_SIZE /**/
@@ -58,6 +50,7 @@
 #undef R__HAS_QT5WEB  /**/
 #define R__HAS_DAVIX  /**/
 #define R__HAS_DATAFRAME /**/
+#define R__HAS_ROOT7 /**/
 #undef R__LESS_INCLUDES /**/
 #undef R__HAS_TBB /**/
 
@@ -71,12 +64,12 @@
 #define R__HAS_DEFAULT_ZLIB  /**/
 #undef R__HAS_DEFAULT_LZMA  /**/
 #undef R__HAS_DEFAULT_ZSTD  /**/
-#undef R__HAS_CLOUDFLARE_ZLIB /**/
+#define R__HAS_CLOUDFLARE_ZLIB /**/
 
-#define R__HAS_TMVACPU /**/
+#undef R__HAS_TMVACPU /**/
 #undef R__HAS_TMVAGPU /**/
 #undef R__HAS_CUDNN /**/
-#undef R__HAS_PYMVA /**/
+#define R__HAS_PYMVA /**/
 #undef R__HAS_RMVA /**/
 
 #undef R__HAS_URING /**/
